@@ -733,7 +733,7 @@ gtk_button_construct_child (GtkButton *button)
       image = g_object_ref (priv->image);
       if (image->parent)
 	gtk_container_remove (GTK_CONTAINER (image->parent), image);
-      
+
       priv->image = NULL;
     }
   
@@ -1004,11 +1004,40 @@ gtk_button_unmap (GtkWidget *widget)
   GTK_WIDGET_CLASS (gtk_button_parent_class)->unmap (widget);
 }
 
+static void
+gtk_button_update_image_spacing (GtkButton *button)
+{
+  GtkButtonPrivate *priv = GTK_BUTTON_GET_PRIVATE (button);
+  GtkWidget *child; 
+  gint spacing;
+
+  /* Keep in sync with gtk_button_construct_child,
+   * we only want to update the spacing if the box 
+   * was constructed there.
+   */
+  if (!button->constructed || !priv->image)
+    return;
+
+  child = GTK_BIN (button)->child;
+  if (GTK_IS_ALIGNMENT (child))
+    {
+      child = GTK_BIN (child)->child;
+      if (GTK_IS_BOX (child))
+        {
+          gtk_widget_style_get (GTK_WIDGET (button),
+                                "image-spacing", &spacing,
+                                NULL);
+
+          gtk_box_set_spacing (GTK_BOX (child), spacing);
+        }
+    }   
+}
+
 static void
 gtk_button_style_set (GtkWidget *widget,
 		      GtkStyle  *prev_style)
 {
-  gtk_button_construct_child (GTK_BUTTON (widget));
+  gtk_button_update_image_spacing (GTK_BUTTON (widget));
 }
 
 static void
@@ -1907,11 +1936,18 @@ gtk_button_grab_notify (GtkWidget *widget,
 			gboolean   was_grabbed)
 {
   GtkButton *button = GTK_BUTTON (widget);
+  gboolean save_in;
 
   if (!was_grabbed)
     {
-      button->in_button = FALSE;
+      save_in = button->in_button;
+      button->in_button = FALSE; 
       gtk_real_button_released (button);
+      if (save_in != button->in_button)
+        {
+          button->in_button = save_in;
+          gtk_button_update_state (button);
+        }
     }
 }
 
@@ -1938,6 +1974,9 @@ gtk_button_set_image (GtkButton *button,
 
   priv = GTK_BUTTON_GET_PRIVATE (button);
 
+  if (priv->image && priv->image->parent)
+    gtk_container_remove (GTK_CONTAINER (priv->image->parent), priv->image);
+
   priv->image = image;
   priv->image_is_stock = (image == NULL);
 
